@@ -201,18 +201,31 @@ class EdhocCredentials(credentials._Objectish):
         else:
             self.own_key = None
 
-        if (own_cred is None) != (own_cred_style is None) or (own_cred is None) != (
-            self.own_key is None
-        ):
-            raise credentials.CredentialsLoadError(
-                "If own credentials are given, all of own_cred, own_cred_style and private_key(_path) need to be given"
-            )
-
         if own_cred_style is None:
             self.own_cred_style = None
         else:
             self.own_cred_style = edhoc.OwnCredStyle(own_cred_style)
 
+        if self.own_cred == {"unauthenticated": True}:
+            if self.own_key is not None or self.own_cred_style not in (
+                None,
+                OwnCredStyle.ByValue,
+            ):
+                raise credentials.CredentialsLoadError(
+                    "For local unauthenticated use, no key can be give, and own_cred_style needs to be by-value or absent"
+                )
+            self.own_key = CoseKeyForEdhoc.generate()
+            # FIXME: kid and subj should rather not be sent, but lakers insists on their presence
+            self.own_cred = self.own_key.as_ccs(kid=b"?", subject="")
+            self.own_cred_style = OwnCredStyle.ByValue
+        else:
+            if (own_cred is None) != (own_cred_style is None) or (own_cred is None) != (
+                self.own_key is None
+            ):
+                raise credentials.CredentialsLoadError(
+                    "If own credentials are given, all of own_cred, own_cred_style and private_key(_path) need to be given"
+                )
+
         # FIXME: This is only used on the client side, and expects that all parts (own and peer) are present
         self._established_context = None
 
@@ -223,13 +236,13 @@ class EdhocCredentials(credentials._Objectish):
             # Only recognizing CCS so far
             return None
 
-        if id_cred_peer == cbor2.dumps(self.peer_cred[14], canonical=True):
+        if id_cred_peer == cbor2.dumps(self.peer_cred, canonical=True):
             # credential by value
             return cbor2.dumps(self.peer_cred[14], canonical=True)
 
         # cnf / COS_Key / kid, should be present in all CCS
         kid = self.peer_cred[14][8][1].get(2)
-        if kid is not None and id_cred_peer == kid:
+        if kid is not None and id_cred_peer == cbor2.dumps({4: kid}, canonical=True):
             # credential by kid
             return cbor2.dumps(self.peer_cred[14], canonical=True)
 
@@ -278,17 +291,19 @@ class EdhocCredentials(credentials._Objectish):
         key_i = self.own_key.d
 
         logger.debug("EDHOC responder sent message_2 with ID_CRED_R = %r", id_cred_r)
-        if self.peer_cred == {"unauthenticated": True}:
+        if self.peer_cred_is_unauthenticated():
             # Not doing further checks (eg. for trailing bytes) or re-raising: This
             # was already checked by lakers
             parsed = cbor2.loads(id_cred_r)
 
-            if not isinstance(parsed, dict):
+            if 14 not in parsed:
                 raise credentials.CredentialsMissingError(
-                    "Peer presented credential-by-reference when no credential was pre-agreed"
+                    "Peer presented credential-by-reference (or anything else that's not a KCCS) when no credential was pre-agreed"
                 )
 
-            cred_r = id_cred_r
+            # We could also pick the [14] out of the serialized stream and
+            # treat it as an opaque item, but cbor2 doesn't really do serialized items.
+            cred_r = cbor2.dumps(parsed[14], canonical=True)
         else:
             # We could look into id_cred_r, which is a CBOR encoded
             # byte string, and could start comparing ... but actually
@@ -335,6 +350,10 @@ class _EdhocContextBase(
         return outer_message, request_id
 
     def _make_ready(self, edhoc_context, c_ours, c_theirs):
+        # only in lakers >= 0.5 that is present
+        if hasattr(edhoc_context, "completed_without_message_4"):
+            edhoc_context.completed_without_message_4()
+
         # FIXME: both should offer this
         if (
             isinstance(edhoc_context, lakers.EdhocResponder)
